import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Arrays;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import com.mindfusion.charting.FunctionSeries;
import com.mindfusion.charting.GridType;
import com.mindfusion.charting.PerSeriesStyle;
import com.mindfusion.charting.SeriesStyle;
import com.mindfusion.charting.swing.LineChart;
import com.mindfusion.drawing.Brush;
import com.mindfusion.drawing.DashStyle;
import com.mindfusion.drawing.SolidBrush;


public class ScriptedSeries extends JFrame
{
	LineChart lineChart;

	final String defaultFormula = "Sin(x / 10) * Cos(x) * 20";
	FunctionSeries functionSeries;

	void initChart()
	{
		lineChart = new LineChart();

		// parse the default formula
		try
		{
			functionSeries = new FunctionSeries(
				defaultFormula, 100000, 10000);
			functionSeries.setTitle("Series 1");
			lineChart.getSeries().add(functionSeries);

			//FunctionSeries can also be initialized with lambda expression
			//lineChart.getSeries().add(
			//	new FunctionSeries(x -> Math.sin(x / 10) * Math.cos(x) * 20 + 40, 100000, 10000));
		}
		catch (Exception e)
		{
			// possible parse errors
			e.printStackTrace();
		}

		// stroke each series using different color
		List<Brush> fills = Arrays.asList(
			new SolidBrush(new Color(0,52,102)),
			new SolidBrush(new Color(102, 154, 204)),
			new SolidBrush(new Color(206, 0, 0)),
			new SolidBrush(new Color(192, 192, 192)),
			new SolidBrush(new Color(125, 125, 125)));

		List<Brush> strokes = Arrays.asList(
			new SolidBrush(new Color(0,52,102)),
			new SolidBrush(new Color(102, 154, 204)),
			new SolidBrush(new Color(206, 0, 0)),
			new SolidBrush(new Color(192, 192, 192)),
			new SolidBrush(new Color(125, 125, 125)));
		
		SeriesStyle style = new PerSeriesStyle(fills, strokes, null, null);
		lineChart.getPlot().setSeriesStyle(style);

		// set initial visible range
		lineChart.getXAxis().setInterval(50.0);
		lineChart.getXAxis().setMinValue(0.0);
		lineChart.getXAxis().setMaxValue(500.0);
		lineChart.getYAxis().setMinValue(-100.0);
		lineChart.getYAxis().setMaxValue(100.0);

		// set appearance properties
		lineChart.setGridType(GridType.Crossed);
		lineChart.getTheme().setGridLineColor(new Color(225, 225, 225));
		lineChart.getTheme().setGridLineStyle(DashStyle.Dash);
		lineChart.getPlot().setBackground(new SolidBrush(Color.WHITE));
	}

	void initFrame()
	{
		setTitle("MindFusion.Charting sample: Scripted Series");
		setSize(650, 400);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		initChart();
		getContentPane().add(lineChart, BorderLayout.CENTER);

		txtFormula = new JTextField(defaultFormula);
		txtFormula.setMinimumSize(new Dimension(150, 30));
		btnExecute = new JButton("Execute");
		btnExecute.addActionListener(e->
		{
			try
			{
				lineChart.getSeries().remove(0);
				functionSeries = new FunctionSeries(
					txtFormula.getText(), 100000, 10000);
				functionSeries.setTitle("Series 1");
				lineChart.getSeries().add(functionSeries);
				lineChart.repaint();
			}
			catch (Exception e1)
			{
				// possible parse errors
				e1.printStackTrace();
			}
		});
		
		pnlControls = new JPanel();
		pnlControls.setLayout(new BoxLayout(pnlControls, BoxLayout.X_AXIS));
		pnlControls.add(txtFormula);
		pnlControls.add(btnExecute);
		
		getContentPane().add(pnlControls, BorderLayout.SOUTH);
		setVisible(true);
	}
	
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					ScriptedSeries frame = new ScriptedSeries();
					frame.initFrame();
				}
				catch (Exception exp)
				{
				}
			}
		});
	}

	private JTextField txtFormula;
	private JButton btnExecute;
	private JPanel pnlControls;

	private static final long serialVersionUID = 1L;
}
